import vector,color,random,bisect

class PriorityQueue:
	def __init__(self):
		self.queue = []
	def append(self,data,priority):
		"""append a new element to the queue according to its priority"""
		bisect.insort(self.queue,(priority,data))
	def pop(self,n):
		"""pop the hightest element of the queue. The n argument is
		here to follow the standard queue protocol """
		return self.queue.pop(0)[1]



class Creature:
    "A simple creature"
    def __init__(self):        
      self.skin_color = color.Color()
      self.position = vector.Vector(random.randrange(0,800),random.randrange(0,600),random.randrange(0,400))
      self.neighbours = []
      self.velocity = vector.Vector()
    
    def add_neighbour(self,neighbour):
      self.neighbours.append(neighbour)
      
    def close_creatures(self):
        pq = PriorityQueue()
        closest = []
        for n in self.neighbours:
          pq.append(n, self.position.distance_to(n.position))
        for i in range(0,5):
          closest.append(pq.pop(1))
        return closest
      
    def distance_velocity(self):
      distance_velocity = vector.Vector()
      for other_critter in self.neighbours:
        if self.position.distance_to(other_critter.position) < 20:
          distance_velocity = distance_velocity - (other_critter.position - self.position)
      return distance_velocity
    
    def center_velocity(self):
      center_velocity = vector.Vector()
      close_neighbours = self.close_creatures()
      for neighbour in close_neighbours:
        center_velocity = center_velocity + neighbour.position
      center_velocity = center_velocity / len(close_neighbours)
      return (center_velocity - self.position) / 150
    
    def similar_velocity(self):
      similar_velocity = vector.Vector()
      close_neighbours = self.close_creatures()
      for neighbour in close_neighbours:
        similar_velocity = similar_velocity + neighbour.velocity
      similar_velocity = similar_velocity / len(close_neighbours)
      return (similar_velocity - self.velocity) / 8
    
    def limit_velocity(self):
      max_speed = 30
      speed = self.velocity.size()
      if speed > max_speed:
        self.velocity= (self.velocity / speed ) * max_speed

    
    def bounding_box_velocity(self,width, height):
      v = vector.Vector()
      force=7
      if self.position.x < 0:
        v.x = force
      elif self.position.x > width:
        v.x = -1 * force
      
      if self.position.y < 0:
        v.y = force
      elif self.position.y > width:
        v.y = -1 * force
      
      return v
      
    def fitness(self,background_color):
      return self.skin_color.distance_to(background_color)
        
    def mate(self,creature):
      new_color = self.skin_color.mean(creature.skin_color)
      child = Creature()
      child.skin_color= new_color
      return child
      
    def mutate(self):
      self.skin_color.abberate()